/*===========================================================*/
/*
 * Parallel Recursive Spectral Bisection -
 * Based on the sequential version provided by Horst Simon
 * -ravi
 */
/*===========================================================*/

#include <stdio.h>
#include "parti.h"

/**********************************************************************/
/* Partitioner interface - The eigen value partitioner takes in       */
/* adajacency list information & returns processor assignment list    */
/*                                                                    */
/* row_ind - list of initial indices assigned (in blocks)             */
/* csr_col - list of indices associated with each row_ind             */
/* csr_ptr - pointer into csr_col                                     */
/* nrow_ind - number of local indices assigned                        */
/* ncols     -s size of csr_col                                       */
/* color   - processor assignment array returned by the partitioner.  */
/*                                                                    */
/* --Ravi                                                             */
/**********************************************************************/

/*====================================================================*/
void parallel_partitioner_old(row_ind, nrow_ind, xadj, adj,  xcount, color)
int *row_ind, *xadj, *adj, nrow_ind, xcount, *color;
{

   int pfac;


/*   pfac = nodedim();	*/
   if(PARTI_mynode == 0)
   printf("\nMAPPER[%d - %d]: ",PARTI_mynode,  xcount);

   eigen_partitioner(row_ind, nrow_ind, xadj, adj,  xcount, pfac, color);

}
/*====================================================================*/



/* fortran callable */
void
parallel_partitioner_old_(row_ind, nrow_ind, xadj, adj, xcount, color)
int *row_ind, *xadj, *adj, *nrow_ind,  *xcount, *color;
{
  int i;

   /** for stupid fortran **/
   for(i=0; i<*nrow_ind; i++) {
       row_ind[i]--;
   }

  parallel_partitioner_old(row_ind, *nrow_ind, xadj, adj,  *xcount, color);

   /** for stupid fortran **/
   for(i=0; i<*nrow_ind; i++) {
       row_ind[i]++;
   }
}
/*====================================================================*/


void parallel_partitioner_(row_ind, nrow_ind, xadj, adj,  itable)
int *row_ind, *xadj, *adj, *nrow_ind;
int *itable;
{
  int i;
  TTABLE *ntable;
  int xcount;
  int pfac, *color;
   TTABLE *init_ttable_with_proc();

   /** for stupid fortran  -NO NEED TO CHANGE xadj & adj GENERATED BY C CODE**/
   for(i=0; i<*nrow_ind; i++) {
       row_ind[i]--;
   }
/*   pfac = nodedim();	*/
   xcount = xadj[*nrow_ind];
   if(PARTI_mynode == 0)
   printf(" Parallel Partitioner[%d]: ncols %d \n",PARTI_mynode,  xcount);

   color = (int *)malloc(sizeof(int)*(*nrow_ind));
   eigen_partitioner(row_ind, *nrow_ind, xadj, adj,  xcount, pfac, color);

   ntable = (TTABLE *) init_ttable_with_proc(1, color, *nrow_ind);

   *itable = (int ) register_table((TTABLE *) ntable);

   /** for stupid fortran **/
   for(i=0; i<*nrow_ind; i++) {
       row_ind[i]++;
   }
}
/*====================================================================*/


void parallel_partitioner(row_ind, nrow_ind, xadj, adj,  ntable)
int *row_ind, *xadj, *adj, nrow_ind;
TTABLE **ntable;
{

   int pfac;
   int *color;
   int xcount;
   TTABLE *init_ttable_with_proc();

/*   pfac = nodedim();	*/
   xcount = xadj[nrow_ind];
   if(PARTI_mynode == 0)
   printf(" Parallel Partitioner[%d]: ncols %d \n",PARTI_mynode,  xcount);

   color = (int *)malloc(sizeof(int)*nrow_ind);
   eigen_partitioner(row_ind, nrow_ind, xadj, adj,  xcount, pfac, color);

   *ntable = (TTABLE *) init_ttable_with_proc(1, color, nrow_ind);
   free(color);

}
/*====================================================================*/
